@@ -499,6 +499,10 @@ private:
             consumer->complete();
             return;
         }
+        if (m_options.get_enable_empty_batches()) {
+            consumer->complete_empty(
+                std::make_shared<watermark_provider_impl>(m_lower_bound_row_ids, m_lower_bound_timestamps));
+        }
 
         // We processed all responses and found no updates. Requesting data again.
         m_resume_part_id = 0;
