@@ -30,13 +30,24 @@ public:
     table_row_event_batch() = default;
 
     /**
+     * Internal API.
      * Constructor.
      *
-     * @param events Events.
+     * @param events Vector of events. Should not be empty.
      */
     explicit table_row_event_batch(std::vector<table_row_event<value_type>> &&events)
         : m_events(std::move(events)) {}
 
+    /**
+     * Internal API.
+     * Constructor.
+     *
+     * @param watermark_provider Watermark provider.
+     */
+    explicit table_row_event_batch(std::shared_ptr<detail::watermark_provider> watermark_provider)
+        : m_watermark_provider(std::move(watermark_provider)) {}
+
+
     /**
      * Gets the events.
      *
@@ -51,9 +62,25 @@ public:
      */
     [[nodiscard]] std::vector<table_row_event<value_type>> &&get_events() && { return std::move(m_events); }
 
+    /**
+     * Gets the watermark for the batch.
+     *
+     * @return Watermark for the batch.
+     */
+    [[nodiscard]] continuous_query_watermark get_watermark() const {
+        if (m_events.empty()) {
+            return m_watermark_provider->get_watermark(-1);
+        }
+
+        return m_events.rbegin()->get_watermark();
+    }
+
 private:
     /** Table row events. */
-    std::vector<table_row_event<value_type>> m_events;
+    std::vector<table_row_event<value_type>> m_events{};
+
+    /** Watermark provider. Required when batch contains no events. */
+    std::shared_ptr<detail::watermark_provider> m_watermark_provider{};
 };
 
 } // namespace ignite
